package com.rightanglegames.backtothepast;
import it.randomtower.engine.ME;
import it.randomtower.engine.ResourceManager;

import java.io.IOException;

import org.newdawn.slick.AppGameContainer;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.util.Log;


public class BackToThePast extends StateBasedGame {

	public static final int WIDTH = 640;
	public static final int HEIGHT = 480;
	public static final int TARGET_FRAME_RATE = 60;
	public static final int TILESIZE = 40;

	// static finals for the game state ids, defined here to avoid duplicate gamestate definitions
	public static final int SPLASH_SCREEN_STATE = 1;
	public static final int TITLE_STATE = 2;
	public static final int INGAME_STATE = 3;
	public static final int NEXTLEVEL_STATE = 4;
	public static final int GAMEOVER_STATE = 5;
	public static final int CREDITS_STATE = 6;
	public static final int INSTRUCTIONS_STATE = 7;
	public static final int OPTIONS_STATE = 8;
	public static final int EDITOR_STATE = 9;
	
	// static finals for the depth layers in the game, defined here to not get confused :-)
	public static final int BACKGROUND_LAYER = 0;
	public static final int PARALLAX_1_LAYER = 5;
	public static final int PARALLAX_2_LAYER = 10;
	public static final int TILES_LAYER = 15;
	public static final int ENEMIES_LAYER = 20;
	public static final int PLAYER_LAYER = 25;
	public static final int OBJECT_LAYER = 30;
	public static final int EFFECTS_LAYER = 35;
	public static final int HUD_LAYER = 100;
	
	/** volume for SFX and music */
	public static float musicVolume = 1.0f;
	public static float sfxVolume = 1.0f;
	
	public static boolean wizardMode = false;
	
	public static boolean ressourcesInited = false;
	
	
	private AppGameContainer container;
	
	public BackToThePast() {
		super("Back to the past");
		Globals.game = this;

//		ME.debugEnabled = true;
		ME.keyToggleDebug = Input.KEY_1;
		ME.debugWindowAtTop = false;
		
	}
	

	@Override
	public void initStatesList(GameContainer container) throws SlickException {
		if (container instanceof AppGameContainer) {
			this.container = (AppGameContainer) container;
		}
		ME.setTargetFrameRate(container, TARGET_FRAME_RATE);
		
		addState(new SplashScreen(SPLASH_SCREEN_STATE));
		addState(new TitleState(TITLE_STATE));
//		addState(new CreditsState());
//		addState(new InstructionsState());
		addState(new IngameState(INGAME_STATE));
		addState(new EditorState(EDITOR_STATE));
//		addState(new NextLevelState());
		addState(new GameOverState(GAMEOVER_STATE));
//		addState(new OptionsState());
		
		applyGameOptions();
	}

	/**
	 * @see org.newdawn.slick.state.StateBasedGame#keyPressed(int, char)
	 */
	public void keyPressed(int key, char c) {
		super.keyPressed(key, c);

		if (key == Input.KEY_F4) {
			if (container != null) {
				try {
					container.setFullscreen(!container.isFullscreen());
					if (container.isFullscreen())
						container.setMouseGrabbed(true);
					else
						container.setMouseGrabbed(false);
				} catch (SlickException e) {
					Log.error(e);
				}
			}
		}
	}

	private void applyGameOptions() {
		ResourceManager.setMusicVolume(1.0f);
		ResourceManager.setSfxVolume(1.0f);
	}

	public static void initRessources() throws SlickException {
		if (ressourcesInited)
			return;
		try {
			ResourceManager.loadResources("res/resources.xml");
		} catch (IOException e) {
			Log.error("failed to load ressource file 'res/resources.xml': "
					+ e.getMessage());
			throw new SlickException("Ressource loading failed!");
		}
		
		ressourcesInited = true;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length == 1 && args[0].equals("wizard"))
			wizardMode = true;
		
		try {
//			Log.setVerbose(false);
			AppGameContainer container = new AppGameContainer(new BackToThePast());
			container.setDisplayMode(WIDTH, HEIGHT, false);
//			container.setShowFPS(false);
			// switch off mouse cursor
			container.start();
		} catch (SlickException e) {
			e.printStackTrace();
		}
	}
}
