package com.rightanglegames.backtothepast;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.SlickException;

import com.rightanglegames.backtothepast.weapons.Weapon;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;
import it.randomtower.engine.entity.PhysicsEntity;

public class Crate extends PhysicsEntity {
	public static final String CRATE = "crate";
	
	private boolean moving = false;
	private int pushing = 0;
	private int timer = 10;
	/** crate's speed when being pushed. Shouldn't be faster than player's movement speed */
	private int movement = 2;

	public Crate(float x, float y) {
		super(x, y);
		name = "crate";
		depth = BackToThePast.OBJECT_LAYER;
		currentImage = ResourceManager.getImage("crate");
		addType(CRATE, SOLID);
		setHitBox(0, 0, currentImage.getWidth(), currentImage.getHeight());
	}

	public void update(GameContainer container, int delta)
			throws SlickException {
		// update possible animation stuff
		// super.update(container, delta);
		motion(false, true);
		gravity(delta);

		// check if player pushes the crate
		pushing = 0;
		if (Globals.player.pushesLeft()
				&& this.collideWith(Globals.player, x + 1, y) != null)
			pushing = -1;
		if (Globals.player.pushesRight()
				&& this.collideWith(Globals.player, x - 1, y) != null)
			pushing = 1;
		// if we're being pushed, count the timer down
		if (pushing == -1) {
			if (timer > 0) {
				timer--;
			}
			// start moving if timer has hit bottom
			if (timer < 1) {
				moving = true;
			}
		}

		if (pushing == 1) {
			if (timer > 0) {
				timer--;
			}
			if (timer < 1) {
				moving = true;
			}
		}

		// if we're moving
		if (moving) {
			// if we're still being pushed, set our speed
			if (pushing != 0) {
				speed.x = movement * pushing;
			}
			// update our position
			motion(true, false);
			// update the player position
			Globals.player.motion(true, false);

			// no longer being pushed
			if (pushing == 0) {
				timer = 10;
				moving = false;
			}
		}

		// slide to a stop if we're not moving
		if (!moving) {
			friction(true, false);
			motion(true, false);
		}
	}
	
	@Override
	public void collisionResponse(Entity other) {
		if (other.isType(Weapon.WEAPON)) {
			// we are destroyed
			this.destroy();
		}
	}
}
