package com.rightanglegames.backtothepast;

import java.util.ArrayList;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.gui.AbstractComponent;
import org.newdawn.slick.gui.ComponentListener;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;
import org.newdawn.slick.util.Log;

import com.rightanglegames.backtothepast.enemies.DummyRobot;
import com.rightanglegames.backtothepast.enemies.FlyingRobot;
import com.rightanglegames.backtothepast.enemies.WalkingRobot;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.World;

public class EditorState extends World implements ComponentListener {

	private static final int MAXLEVEL = 99;
	private static final int MAXSIZE = 200;

	private static final int TEST_LEVEL_NO = 999;
	
	private Color background = new Color(127,127,127);
	private int tilex = 0, tiley = 0;
	private float mousex = 0, mousey = 0;
	
	private int levelNo = 1;
	
	private int upperLeftX = 0, upperLeftY = 0;
	
	private EditorTile[][] map = new EditorTile[MAXSIZE][MAXSIZE];
	private ArrayList<String> script;
	private ArrayList<String> description;

	private EditorTile[] drawables;
	private int drawable = 0;
	private boolean leftMouseButtonDown = false, rightMouseButtonDown = false;
	
	private Level wip = new Level();
	
	public EditorState(int id) {
		super(id);
	}

	public void init(GameContainer container, StateBasedGame game) throws SlickException {
		BackToThePast.initRessources();
		super.init(container, game);
		prepareAvailableTiles();
	}

	private void prepareAvailableTiles() {
		drawables = new EditorTile[25];
		int i = 0;
		drawables[i++] = new EditorTile("Block", "=", ResourceManager.getSpriteSheet(Block.BLOCKS), 0, 0);
		drawables[i++] = new EditorTile("Acid", "~", ResourceManager.getSpriteSheet(Acid.ACID), 0, 0);
		drawables[i++] = new EditorTile("Mage", "P", ResourceManager.getSpriteSheet(Mage.MAGE), 0, 0);
		drawables[i++] = new EditorTile("Exit", "X", ResourceManager.getSpriteSheet(LevelExit.LEVEL_EXIT), 0, 0);
		drawables[i++] = new EditorTile("Crate", "#", ResourceManager.getImage(Crate.CRATE));
		drawables[i++] = new EditorTile("Turret up", "^", ResourceManager.getSpriteSheet("turretup"), 1, 0);
		drawables[i++] = new EditorTile("Turret down", "v", ResourceManager.getSpriteSheet("turretdown"), 1, 0);
		drawables[i++] = new EditorTile("Dummy robot", "r", ResourceManager.getImage(DummyRobot.DUMMYROBOT));
		drawables[i++] = new EditorTile("Walking robot", "w", ResourceManager.getImage(WalkingRobot.WALKINGROBOT));
		drawables[i++] = new EditorTile("Flying robot", "f", ResourceManager.getSpriteSheet(FlyingRobot.FLYINGROBOT), 0, 0);
		drawables[i++] = new EditorTile("M", "M", ResourceManager.getSpriteSheet("MAGICK"), 0, 0);
		drawables[i++] = new EditorTile("A", "A", ResourceManager.getSpriteSheet("MAGICK"), 1, 0);
		drawables[i++] = new EditorTile("G", "G", ResourceManager.getSpriteSheet("MAGICK"), 2, 0);
		drawables[i++] = new EditorTile("I", "I", ResourceManager.getSpriteSheet("MAGICK"), 3, 0);
		drawables[i++] = new EditorTile("C", "C", ResourceManager.getSpriteSheet("MAGICK"), 4, 0);
		drawables[i++] = new EditorTile("K", "K", ResourceManager.getSpriteSheet("MAGICK"), 5, 0);
		drawables[i++] = new EditorTile("Waypoint 1", "1", ResourceManager.getSpriteSheet("waypoints"), 0, 0);
		drawables[i++] = new EditorTile("Waypoint 2", "2", ResourceManager.getSpriteSheet("waypoints"), 1, 0);
		drawables[i++] = new EditorTile("Waypoint 3", "3", ResourceManager.getSpriteSheet("waypoints"), 2, 0);
		drawables[i++] = new EditorTile("Waypoint 4", "4", ResourceManager.getSpriteSheet("waypoints"), 3, 0);
		drawables[i++] = new EditorTile("Waypoint 5", "5", ResourceManager.getSpriteSheet("waypoints"), 4, 0);
		drawables[i++] = new EditorTile("Waypoint 6", "6", ResourceManager.getSpriteSheet("waypoints"), 5, 0);
		drawables[i++] = new EditorTile("Waypoint 7", "7", ResourceManager.getSpriteSheet("waypoints"), 6, 0);
		drawables[i++] = new EditorTile("Waypoint 8", "8", ResourceManager.getSpriteSheet("waypoints"), 7, 0);
		drawables[i++] = new EditorTile("Waypoint 9", "9", ResourceManager.getSpriteSheet("waypoints"), 8, 0);
	}


	public void enter(GameContainer container, StateBasedGame game) throws SlickException {
		super.enter(container, game);
		Globals.testLevel = null;
	}
	
	public void render(GameContainer container, StateBasedGame game, Graphics g) throws SlickException {
		// render all entities
		g.setBackground(background);
		super.render(container, game, g);
		drawMap(container, game, g);
		drawInfo(container, game, g);
	}

	private void drawMap(GameContainer container, StateBasedGame game, Graphics g) throws SlickException {
		int w = BackToThePast.WIDTH / BackToThePast.TILESIZE;
		int h = BackToThePast.HEIGHT / BackToThePast.TILESIZE;

		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				if (map[x+upperLeftX][y+upperLeftY] != null)
					map[x+upperLeftX][y+upperLeftY].render(g, x * BackToThePast.TILESIZE, y * BackToThePast.TILESIZE);
			}
		}
	}

	private void drawInfo(GameContainer container, StateBasedGame game, Graphics g) {
		int x = 0, y = 0;
		g.resetFont(); // switch back to system font
		g.drawString("level " + levelNo + ", tile " + tilex + ":" + tiley, x, y); y+=10;
		g.drawString("drawing: " + drawables[drawable].getName(), x, y);
		
	}


	public void update(GameContainer container, StateBasedGame game, int delta) throws SlickException {
		// update all entities
		super.update(container, game, delta);
	}
	
	/**
	 * @see org.newdawn.slick.InputListener#keyPressed(int, char)
	 */
	public void keyPressed(int key, char c) {
		if (key == Input.KEY_ESCAPE) {
			// go back to title state
			Globals.game.enterState(BackToThePast.TITLE_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
		}
		if (key == Input.KEY_RIGHT) {
			upperLeftX++;
		}
		if (key == Input.KEY_LEFT) {
			upperLeftX--;
			if (upperLeftX < 0)
				upperLeftX = 0;
		}
		if (key == Input.KEY_UP) {
			upperLeftY--;
			if (upperLeftY < 0)
				upperLeftY = 0;
		}
		if (key == Input.KEY_DOWN) {
			upperLeftY++;
		}
		if (key == Input.KEY_D) {
			// TODO enter description
		}
		if (key == Input.KEY_S) {
			// TODO enter script
		}
		if (key == Input.KEY_M) {
			// TODO show minimap
		}
		if (key == Input.KEY_ADD) {
			levelNo++;
			if (levelNo > MAXLEVEL)
				levelNo = 0;
		}
		if (key == Input.KEY_MINUS) {
			levelNo--;
			if (levelNo < 0)
				levelNo = MAXLEVEL;
		}
		if (key == Input.KEY_F1) {
			// load level
			try {
				this.loadLevel(levelNo);
			} catch (SlickException e) {
				e.printStackTrace();
			}
		}
		if (key == Input.KEY_F2) {
			// save level
			this.saveLevel(levelNo);
		}
		if (key == Input.KEY_T) {
			// save test level, store it in globals and enter game state
			this.saveLevel(TEST_LEVEL_NO);
			try {
				Globals.testLevel = Level.load(TEST_LEVEL_NO);
				Globals.game.enterState(BackToThePast.INGAME_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
			} catch (SlickException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		recalculatePosition();
	}

	public void mouseWheelMoved(int change) {
		if (change < 0) {
			drawable --;
			if (drawable < 0)
				drawable = 0;
		}
		if (change > 0) {
			drawable ++;
			if (drawable >= drawables.length)
				drawable = drawables.length-1;
		}
	}
	/**
	 * @see org.newdawn.slick.InputListener#mouseClicked(int, int, int, int)
	 */
	public void mousePressed(int button, int x, int y) {
		if (button == 0) {
			leftMouseButtonDown = true;
			addCurrentDrawable();
		} else if (button == 1) {
			rightMouseButtonDown = true;
			deleteCurrentDrawable();
		}
	}
	
	public void mouseReleased(int button, int x, int y) {
		if (button == 0) {
			leftMouseButtonDown = false;
		} else if (button == 1) {
			rightMouseButtonDown = false;
		}
	}
	
	private void addCurrentDrawable() {
		// add current drawable at tilex, tiley
		if (map[tilex][tiley] == null)
			map[tilex][tiley] = drawables[drawable];
	}
	
	private void deleteCurrentDrawable() {
		// delete current drawable at tilex, tiley
		if (map[tilex][tiley] != null)
			map[tilex][tiley] = null;
	}

	@Override
	public void componentActivated(AbstractComponent source) {
		// TODO Auto-generated method stub
		
	}

	private void recalculatePosition() {
		tilex = (int) (mousex / BackToThePast.TILESIZE) + upperLeftX;
		tiley = (int) (mousey / BackToThePast.TILESIZE) + upperLeftY;
	}

	/**
	 * @see org.newdawn.slick.BasicGame#mouseMoved(int, int, int, int)
	 */
	public void mouseMoved(int oldx, int oldy, int newx, int newy) {
		mousex = newx;
		mousey = newy;
		recalculatePosition();
		if (leftMouseButtonDown)
			addCurrentDrawable();
		else if (rightMouseButtonDown)
			deleteCurrentDrawable();
	}
	
	public void testLevel() {
		
	}

	private void loadLevel(int levelno) throws SlickException {
		// first load the original level structure
		Level lev = Level.load(levelno);
		if (lev.getWidthInTiles() >= MAXSIZE || lev.getHeightInTiles() >= MAXSIZE)
			throw new SlickException("loaded level too large for level editor!");
		map = new EditorTile[MAXSIZE][MAXSIZE];
		for (int x = 0; x < lev.getWidthInTiles(); x++) {
			for (int y = 0; y < lev.getHeightInTiles(); y++) {
				if (lev.getLines().get(y).charAt(x) != '.') {
					String mapChar = Character.toString(lev.getLines().get(y).charAt(x));
					EditorTile d = this.getDrawable(mapChar);
					this.map[x][y] = d;
				}
			}
		}
		this.description = lev.getDescription();
		this.script = lev.getScript();
	}

	
	private EditorTile getDrawable(String mapChar) {
		for (EditorTile drawable : drawables) {
			if (mapChar.equals(drawable.getMapChar()))
				return drawable;
		}
		Log.error("Leveleditor during level load: no drawable for map char '" + mapChar + "' found!!!");
		return null;
	}

	private boolean saveLevel(int levelno) {
		wip = new Level();
		wip.setLevelNo(levelno);
		wip.setLines(this.generateLinesFromMap());
		wip.setDescription(this.description);
		wip.setScript(this.script);
		return wip.saveToFile();
	}

	private ArrayList<String> generateLinesFromMap() {
		int width = 0, height = 0;
		// first calculate map size
		for (int x = 0; x < MAXSIZE; x++) {
			for (int y = 0; y < MAXSIZE; y++) {
				if (map[x][y] != null) {
					if (x > width)
						width = x;
					if (y > height)
						height = y;
				}
			}
		}
		// now second loop to generate lines
		ArrayList<String> lines = new ArrayList<String>();
		for (int y = 0; y <= height; y++) {
			StringBuffer buf = new StringBuffer();
			for (int x = 0; x <= width; x++) {
				if (map[x][y] != null)
					buf.append(map[x][y].getMapChar());
				else
					buf.append(".");
			}
			lines.add(buf.toString());
		}
		return lines;
	}
}
