package com.rightanglegames.backtothepast;

import it.randomtower.engine.ResourceManager;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

public class GameOverState extends EnhancedWorld {

	private static final String CMD_PLAY = "play";
	private static final String CMD_QUIT = "quit";

	private static String message = "Press P to play, ESC to quit";
	private static String yourScoreWas = "Your score was";
	
	private Image youLose, youWin;
	private AngelCodeFont hilda;
	private String lastScore = "";

	public GameOverState(int id) {
		super(id);
		define(CMD_QUIT, Input.KEY_ESCAPE);
		define(CMD_PLAY, Input.KEY_P, Input.KEY_SPACE, Input.KEY_RETURN);
	}

	public void init(GameContainer container, StateBasedGame game) throws SlickException {
		BackToThePast.initRessources();
		super.init(container, game);
		youLose = ResourceManager.getImage("gameover");
		youWin = ResourceManager.getImage("gamewon");
		hilda = ResourceManager.getAngelCodeFont("HildaSonnenschein32");
	}

	public void enter(GameContainer container, StateBasedGame game) throws SlickException {
		super.enter(container, game);
		lastScore = Integer.toString(((IngameState)Globals.game.getState(BackToThePast.INGAME_STATE)).getScore());
		this.clear();
	}
	public void render(GameContainer container, StateBasedGame game, Graphics g) throws SlickException {
		// render all entities
		g.setBackground(Color.black);
		super.render(container, game, g);
		if (Globals.gameWon)
			youWin.draw(0, 0);
		else
			youLose.draw(0,0);
		g.setFont(hilda);
		g.setColor(Color.black);
		if (!Globals.gameWon)
			g.drawString(yourScoreWas, BackToThePast.WIDTH / 2 - hilda.getWidth(yourScoreWas) / 2, 270);
		g.drawString(lastScore, BackToThePast.WIDTH / 2 - hilda.getWidth(lastScore) / 2, 300);
		g.drawString(message, BackToThePast.WIDTH / 2 - hilda.getWidth(message) / 2, 400);
	}

	public void update(GameContainer container, StateBasedGame game, int delta) throws SlickException {
		// update all entities
		super.update(container, game, delta);
		if (check(CMD_PLAY))
			Globals.game.enterState(BackToThePast.INGAME_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
		if (check(CMD_QUIT)) {
			container.exit();
		}
	}
}
