package com.rightanglegames.backtothepast;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;
import org.newdawn.slick.util.pathfinding.AStarPathFinder;

import it.randomtower.engine.Camera;
import it.randomtower.engine.World;
import it.randomtower.engine.entity.Entity;

public class IngameState extends World {
	
	public IngameState(int id) {
		super(id);
	}

	private int levelNo = 1;
	private int score = 0;
	private Level currentLevel = null;
	
	private Camera camera;
	private Color background = new Color(127,127,127);
	private Blender blender;
	private AStarPathFinder pathfinder;
	
	public void init(GameContainer container, StateBasedGame game) throws SlickException {
		BackToThePast.initRessources();
		super.init(container, game);
	}

	public void enter(GameContainer container, StateBasedGame game) throws SlickException {
		super.enter(container, game);
		this.resetGame();
	}


	private void resetGame() throws SlickException {
		Globals.gameWon = false;
		this.setLevelNo(1);
		this.setScore(0);
		this.resetLevel(true);
	}
	
	private void resetLevel(boolean firstStart) throws SlickException {
		int oldMana = 0;
		if (!firstStart) {
			Mage mage = (Mage) this.find(Mage.MAGE);
			if (mage != null)
				oldMana = mage.getMana();
		}
		this.clear();
		this.blender = null;
		// add the hud to the world directly
		this.add(new Hud(1,1), World.ABOVE);
		Level nextLevel = null;
		if (Globals.testLevel != null)
			nextLevel = Globals.testLevel;
		else
			nextLevel = Level.load(getLevelNo());
		
		if (nextLevel == null) {
			// you win the game
			Globals.gameWon = true;
			Globals.game.enterState(BackToThePast.GAMEOVER_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
			return;
		}
		this.setCurrentLevel(nextLevel);
		int tilewidth = this.getCurrentLevel().getWidthInTiles();
		int tileheight = this.getCurrentLevel().getHeightInTiles();
		this.width = tilewidth * BackToThePast.TILESIZE;
		this.height = tileheight * BackToThePast.TILESIZE;
		this.getCurrentLevel().createEntities(this.getCurrentLevel(), tilewidth, tileheight, this);
		pathfinder = new AStarPathFinder(this.getCurrentLevel(), 500, false);
		if (!firstStart) {
			Mage mage = (Mage) this.find(Mage.MAGE);
			if (mage != null)
				mage.setMana(oldMana);
		}
		// add the parallax scrolling background
		this.add(new ParallaxBackground(0, 0, this));
		// check which level we loaded
		this.setLevelNo(this.getCurrentLevel().getLevelNo());
		if (Globals.playerCheated) {
			this.setScore(0);
			// reset score to 0 every new level. cheating is not nice and won't be forgotten ;-)
		}
		Globals.playerDead = false;
		Globals.playerResurrected = false;
		Globals.levelDone = false;
		if (firstStart && this.currentLevel.getDescription() != null && this.currentLevel.getDescription().size() > 0) {
			MessageWindow desc = new MessageWindow(this.currentLevel.getDescription(), 0, 0, true);
			this.add(desc, World.ABOVE);
			Globals.messageWindow = desc;
		} else
			Globals.messageWindow = null;
	}

	public void render(GameContainer container, StateBasedGame game, Graphics g) throws SlickException {
		// render all entities
		g.setBackground(background);
		super.render(container, game, g);
	}

	public void update(GameContainer container, StateBasedGame game, int delta) throws SlickException {
		if (this.camera == null)
			this.setCamera(new Camera(this, this.find(Mage.MAGE), BackToThePast.WIDTH, BackToThePast.HEIGHT));

		// immediately leave as long as the message window is shown
		if (Globals.messageWindow != null) {
			// just update the message window
			Globals.messageWindow.update(container, delta);
			return;
		}
		
		// update all entities
		super.update(container, game, delta);

		if (Globals.playerDead || Globals.playerResurrected || Globals.levelDone) {
			if (blender == null) {
				blender = new Blender(0,0,BackToThePast.WIDTH, BackToThePast.HEIGHT, Color.white, 1000, 1000);
				this.add(blender, World.ABOVE);
			}
			if (blender.finished()) {
				blender = null;
				if (Globals.levelDone) {
					this.setLevelNo(this.getLevelNo() + 1);
					this.increaseScore(1000);
					this.resetLevel(true);
				} else if (Globals.playerResurrected) {
					this.resetLevel(false);
				} else if (Globals.playerDead) {
					if (Globals.testLevel != null) {
						// move to editor state
						Globals.game.enterState(BackToThePast.EDITOR_STATE, null, null);
					} else {
						// move to game over state
						Globals.game.enterState(BackToThePast.GAMEOVER_STATE, null, null);
					}
				}
			}
		}
	}

	/**
	 * @see org.newdawn.slick.InputListener#keyPressed(int, char)
	 */
	public void keyPressed(int key, char c) {
		if (key == Input.KEY_ESCAPE) {
			if (Globals.testLevel != null) {
				// go back to level editor
				Globals.game.enterState(BackToThePast.EDITOR_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
			} else {
			// go back to title state
				Globals.game.enterState(BackToThePast.TITLE_STATE, new FadeOutTransition(Color.white), new FadeInTransition(Color.white));
			}
		}
	}
	
	@Override
	public int getID() {
		return BackToThePast.INGAME_STATE;
	}

	public void setLevelNo(int levelNo) {
		this.levelNo = levelNo;
	}

	public int getLevelNo() {
		return levelNo;
	}

	public void setScore(int score) {
		this.score = score;
	}

	public int getScore() {
		return score;
	}
	
	public void increaseScore(int value) {
		this.score += value;
	}

	public void setCurrentLevel(Level currentLevel) {
		this.currentLevel = currentLevel;
	}

	public Level getCurrentLevel() {
		return currentLevel;
	}

	public AStarPathFinder getPathfinder() {
		return pathfinder;
	}

}
