package com.rightanglegames.backtothepast;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Vector2f;

import com.rightanglegames.backtothepast.weapons.Bullet;
import com.rightanglegames.backtothepast.weapons.Fireball;
import com.rightanglegames.backtothepast.weapons.Laserbeam;

import it.randomtower.engine.ME;
import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;
import it.randomtower.engine.entity.PhysicsEntity;

public class Mage extends PhysicsEntity {
	public static final String MAGE = "mage";
	
	public static final int FIREBALL_DEFAULT_MANA_COST = 10;
	public static final int DEFAULT_MANA_RESTORE = 1;
	public static final int RESURRECTION_MANA_COST = 25;
	
	// constants for the commands
	public static final String CMD_LEFT = "left";
	public static final String CMD_RIGHT = "right";
	public static final String CMD_JUMP = "jump";
	public static final String CMD_FIRE = "fire";
	
	private int moveSpeed = 1;
	private int jumpSpeed = 8;
	private boolean faceRight = true;
	
	private boolean pushesLeft = false;
	private boolean pushesRight = false;
	
	private boolean onGround = false;
	
	// did we jump or just fell off some solid?
	private boolean didJump = false;
	
	//are we walljumping? (0 = no, 1 = left, 2 = right)
	private int wallJumping = 0;
	//can we double jump? (false = no, true = yes)
	private boolean doubleJump = false;
	
	private boolean dead = false;
	private int health = 100;
	private int mana = 100;
	private int fireballManaCost = FIREBALL_DEFAULT_MANA_COST;
	private int manaRestore = DEFAULT_MANA_RESTORE;
	
	private Sound jumpSnd, hitSnd, collectSnd, allCollectedSnd, fireBallSnd;
	
	private boolean canShoot = true;
	
	private int frame = 0;
	private int frameTimer = 0;
	private int frameInterval = 100;	// switch frame every 100 ms if walking

	private String collected = "_______";	// 7 letters for MAGICKA
	
	public Mage(float x, float y) {
		super(x, y);
		depth = BackToThePast.PLAYER_LAYER;
		this.name = MAGE;
		this.addType(Entity.PLAYER);
		sheet = ResourceManager.getSpriteSheet(MAGE);
		currentImage = sheet.getSprite(0, 0);
		width = 40;
		height = 40;

		//set different speeds and such
		gravity = 0.4f;
		maxSpeed = new Vector2f(4, 8);
		Globals.originalPlayerMaxSpeed = new Vector2f(maxSpeed);
		friction = new Vector2f(0.5f, 0.5f);
		//set hitbox
		setHitBox(6, 2, 24, 38);
		defineCommands();
		// set sounds
		//bumpSnd = ResourceManager.getSound("bump");
		jumpSnd = ResourceManager.getSound("jump");
		hitSnd = ResourceManager.getSound("hit");
		collectSnd = ResourceManager.getSound("collect");
		allCollectedSnd = ResourceManager.getSound("allcollected");
		fireBallSnd = ResourceManager.getSound("fireball");
		// create a timer to restore our mana by 2 every second
		this.setAlarm("manaRestore", ME.targetFrameRate / 2, false, true);
		
		// store ourselves in Globals
		Globals.player = this;
	}
	
	private void defineCommands() {
		define(CMD_JUMP, Input.KEY_Y, Input.KEY_Z, Input.KEY_W, Input.KEY_UP);
		define(CMD_FIRE, Input.KEY_X, Input.KEY_LCONTROL, Input.KEY_SPACE);
		define(CMD_LEFT, Input.KEY_LEFT, Input.KEY_A);
		define(CMD_RIGHT, Input.KEY_RIGHT, Input.KEY_D);
		
	}

	public void update(GameContainer container, int delta) throws SlickException {

		previousx = x;
		previousy = y;
		// deal with shooting stuff
		if (canShoot && check(CMD_FIRE) && (mana >= fireballManaCost)) {
			canShoot = false;
			mana -= fireballManaCost;
			this.setAlarm("shootDelay", 30, true, true);
			this.shootFireBall();
		}

		//are we on the ground?
		onGround = false;
		if (collide(SOLID, x, y + 1) != null) 
		{ 
			onGround = true;
			wallJumping = 0;
			doubleJump = true;
			didJump = false;
		}
		
		//set acceleration to nothing
		acceleration.x = 0;
		
		//increase acceleration, if we're not going too fast
		if (check(CMD_LEFT)) {
			this.pushesLeft = true;
			if (speed.x > -maxSpeed.x) {
				acceleration.x = - moveSpeed;
				faceRight = false;
			}
		} else
			this.pushesLeft = false;
		if (check(CMD_RIGHT)) {
			this.pushesRight = true;
			if(speed.x < maxSpeed.x) {
				acceleration.x = moveSpeed;
				faceRight = true;
			}
		} else
			this.pushesRight = false;
		
		//friction (apply if we're not moving, or if our speed.x is larger than maxspeed)
		if ( (! check(CMD_LEFT) && ! check(CMD_RIGHT)) || Math.abs(speed.x) > maxSpeed.x ) {
			friction(true, false);
		}
		
		//jump
		if ( pressed(CMD_JUMP) ) 
		{
			boolean jumped = false;
			
			//normal jump
			if (onGround) { 
				speed.y = -jumpSpeed; 
				jumped = true;
				didJump = true;
				jumpSnd.play();
			}
			
			//wall jump
			if ((collide(SOLID, x - 1, y) != null) && !jumped && wallJumping != 3) 
			{ 
				speed.y = -jumpSpeed;			//jump up
				speed.x = maxSpeed.x * 2;	//move right fast
				wallJumping = 2;			//and set wall jump direction
				jumped = true;				//so we don't "use up" or double jump
				jumpSnd.play();
			}
			//same as above
			if ((collide(SOLID, x + 1, y) != null) && !jumped && wallJumping != 3) 
			{ 
				speed.y = -jumpSpeed; 
				speed.x = - maxSpeed.x * 2;
				wallJumping = 1;
				jumped = true;
				jumpSnd.play();
			}
			
			//set double jump to false
			if (!onGround && !jumped && didJump && doubleJump) { 
				speed.y = -jumpSpeed;
				doubleJump = false;
				//set walljumping to 0 so we can move back in any direction again
				//incase we were wall jumping prior to this double jump.
				//if you don't want to allow walljumping after a double jump, set this to 3.
				wallJumping = 0;
			}
		}
		
		
		//if we ARE walljumping, make sure we can't go back
		if (wallJumping > 0)
		{
			if (wallJumping == 2 && speed.x < 0) { speed.x = 0; }
			if (wallJumping == 1 && speed.x > 0) { speed.x = 0; }
		}
		
		//set the gravity
		gravity(delta);
		
		//make sure we're not going too fast vertically
		//the reason we don't stop the player from moving too fast left/right is because
		//that would (partially) destroy the walljumping. Instead, we just make sure the player,
		//using the arrow keys, can't go faster than the max speed, and if we are going faster
		//than the max speed, descrease it with friction slowly.
		maxspeed(false, true);
		
		//variable jumping (tripple gravity)
		if (speed.y < 0 && !check(CMD_JUMP)) {
			gravity(delta);
			gravity(delta);
		}
		
		
		//set the sprites according to if we're on the ground, and if we are moving or not
		if (onGround) {
			if (speed.x != 0) {
				// update frametimer
				frameTimer -= delta;
				while (frameTimer < 0) {
					frame++;
					frameTimer += frameInterval;
					if (frame > 3)		// 4 frames for mage animation
						frame = 0;
				}
			}
			if (speed.x < 0) {
				currentImage = sheet.getSprite(frame + ((canShoot)?0:4), 0).getFlippedCopy(true, false);
			}
			if (speed.x > 0) {
				currentImage = sheet.getSprite(frame + ((canShoot)?0:4), 0);
			}
			
			if (speed.x == 0) {
				if (faceRight)
					currentImage = sheet.getSprite(0 + ((canShoot)?0:4), 0);
				else
					currentImage = sheet.getSprite(0 + ((canShoot)?0:4), 0).getFlippedCopy(true, false);
			}
		} else {
			if (faceRight)
				currentImage = sheet.getSprite(3 + ((canShoot)?0:4), 0);
			else
				currentImage = sheet.getSprite(3 + ((canShoot)?0:4), 0).getFlippedCopy(true, false);
		}
		
		
		//set the motion. We set this later so it stops all movement if we should be stopped
		motion(true, true);
		
		// do some bounds checking. Our Monk shall not leave the level
		if (x < 0)
			x = 0;
		if (x+width >= world.width)
			x = world.width - width;
		if (y < 0)
			y = 0;
		if (y+height >= world.height)
			y = world.height - height;
		// did we collect a star?
		Entity collectable = collide(Collectable.COLLECTABLE, x, y);
		if (collectable != null) {
			Collectable col = (Collectable) collectable;
			this.pickedUp(col.getLetter());
		}

		// did we collide with acid
		if (collide(Acid.ACID, x, y) != null) {
			Globals.playerDead = true;
			this.visible = false;
		}
		
	}
	
	
	private void pickedUp(String letter) {
		collectSnd.play();
		char what = letter.charAt(0);
		switch(what) {
		case 'M':
			collected = "M" + collected.substring(1);
			break;
		case 'A':
			if (collected.charAt(1) == 'A') {
				collected = collected.substring(0, 6) + "A";
			} else {
				collected = collected.substring(0,1) + "A" + collected.substring(2);
			}
			break;
		case 'G':
			collected = collected.substring(0,2) + "G" + collected.substring(3);
			break;
		case 'I':
			collected = collected.substring(0,3) + "I" + collected.substring(4);
			break;
		case 'C':
			collected = collected.substring(0,4) + "C" + collected.substring(5);
			break;
		case 'K':
			collected = collected.substring(0,5) + "K" + collected.substring(6);
			break;
		}
		mana += 50;
		if (mana > 100)
			mana = 100;
		health += 50;
		if (health > 100)
			health = 100;
		if (allLettersCollected()) {
			allCollectedSnd.play();
			// open the exit
			Entity exit = this.world.find(LevelExit.LEVEL_EXIT);
			if (exit != null)
				exit.triggerAlarm(LevelExit.LEVEL_EXIT);
		}
	}
	
	private boolean allLettersCollected() {
		if (collected.indexOf("_") != -1)
			return false;
		return true;
	}

	private void shootFireBall() {
		int offsetx, ballspeed;
		if (faceRight) {
			offsetx = 20;
			ballspeed = 6;
		} else {
			offsetx = -20;
			ballspeed = -6;
		}
		Fireball fireBall = new Fireball(x+offsetx, y + 10, 60, new Vector2f(ballspeed, 0), 90);
		this.world.add(fireBall);
		fireBallSnd.play();
	}

	public void leftWorldBoundaries() {
		// the player unfortunately left the screen - so we retry
		Globals.playerDead = true;
	}
	
	public void animEnd(){ }

	public boolean pushesLeft() {
		return pushesLeft;
	}

	public boolean pushesRight() {
		return pushesRight;
	}

	public int getHealth() {
		return health;
	}
	
	public int getMana() {
		return mana;
	}
	
	public void setMana(int mana) {
		this.mana = mana;
	}

	@Override
	public void collisionResponse(Entity other) {
		if (other.isType(Bullet.BULLET)) {
			// player looses health
			hitSnd.play();
			this.health -= ((Bullet) other).getDamage();
			if (this.health <= 0) {
				mageDies();
			}
		}
		if (other.isType(Laserbeam.LASERBEAM)) {
			hitSnd.play();
			this.health = 0;
			mageDies();
		}
	}
	
	public void mageDies() {
		if (Globals.playerDead || Globals.playerResurrected)
			return;	// we are already in the dying process
		
		if (this.mana >= RESURRECTION_MANA_COST) {
			// player respawns and can continue
			this.mana -= RESURRECTION_MANA_COST;
			Globals.playerResurrected = true;
		} else {
			// player is really dead now. Bad luck.
			Globals.playerDead = true;
		}
		this.visible = false;
	}

	@Override
	public void alarmTriggered(String name) {
		if (name.equals("shootDelay")) {
			canShoot = true;
		} else if (name.equals("manaRestore")) {
			mana += manaRestore;
			if (mana > 100)
				mana = 100;
		}
	}

	public void setCollected(String collected) {
		this.collected = collected;
	}

	public String getCollected() {
		return collected;
	}

}
