package com.rightanglegames.backtothepast;

import java.util.ArrayList;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;

public class MessageWindow extends Entity {
	public static final String MESSAGEWINDOW = "messageWindow";
	
	public static final String CMD_UP = "up";
	public static final String CMD_DOWN = "down";
	public static final String CMD_SKIP = "skip";
	
	private AngelCodeFont font = null;
	private ArrayList<String> messageLines;
	private Image background;
	private int firstLine = 0;
	private static int linesInWindow = 7;
	private static String scrollMessage = "Use <Up>/<Down> to scroll, <Return> to leave";
	
	public MessageWindow(ArrayList<String> message, int x, int y, boolean centerOnScreen) {
		super(x, y);
		background = ResourceManager.getImage("message");
		this.name = MESSAGEWINDOW;
		this.addType(MESSAGEWINDOW);
		messageLines = message;
		setGraphic(background);
		font = ResourceManager.getAngelCodeFont("HildaSonnenschein32");
		setHitBox(0, 0, width, height);
		depth = BackToThePast.HUD_LAYER;
		if (centerOnScreen) {
			this.x = BackToThePast.WIDTH / 2 - background.getWidth() / 2;
			this.y = BackToThePast.HEIGHT / 2 - background.getHeight() / 2;
		}
		define(CMD_UP, Input.KEY_UP);
		define(CMD_DOWN, Input.KEY_DOWN);
		define(CMD_SKIP, Input.KEY_RETURN);
	}
	
	@Override
	public void update(GameContainer container, int delta) throws SlickException {
		super.update(container, delta);
		if (check(CMD_UP)) {
			if (firstLine > 0)
				firstLine--;
		}
		if (check(CMD_DOWN)) {
			if (firstLine + linesInWindow < messageLines.size())
				firstLine++;
		}
		if (check(CMD_SKIP)) {
			Globals.messageWindow = null;
			this.destroy();
		}
	}
	
	@Override
	public void render(GameContainer container, Graphics g) throws SlickException {
		if (!visible)
			return;
		super.render(container, g);
		g.setColor(Color.black);
		Font oldFont = g.getFont();
		g.setFont(font);
		int startx = (int) x + 20;
		int starty = (int)y + 15;
		for (int i = 0; i < linesInWindow; i++) {
			g.drawString(messageLines.get(firstLine+i), startx, starty);
			starty += 25;
		}
		g.drawString(scrollMessage, startx, y+205);
		g.setColor(Color.white);
		g.setFont(oldFont);
	}
}
