package com.rightanglegames.backtothepast;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.World;
import it.randomtower.engine.entity.Entity;

import java.util.ArrayList;
import java.util.List;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.util.Log;

public class ParallaxBackground extends Entity {

	private ArrayList<Image> buildingImages = new ArrayList<Image>();
	private float worldWidth, worldHeight;

	public ParallaxBackground(float x, float y, World world) {
		super(x, y);
		this.name = "parallaxbackground";
		this.world = world;
		this.worldWidth = world.width;
		this.worldHeight = world.height;
		
		// load all images here
		int i = 1;
		boolean imageExists = true;
		while (imageExists) {
			String building = "building" + Utils.pad(i, 2);
			Image img = ResourceManager.getImage(building);
			if (img != null) {
				buildingImages.add(img);
				i++;
			} else
				imageExists = false;
		}
		generateBackground(world);
	}

	public void update(GameContainer container, int delta) throws SlickException {
		super.update(container, delta);
		List<Entity> buildings = this.world.getEntities(Building.BUILDING);
		if (buildings != null && !buildings.isEmpty()) {
			for (Entity building : buildings) {
				if (building.depth == BackToThePast.PARALLAX_1_LAYER) {
					building.x = building.startx + (int) this.world.camera.cameraX / 2;
				} else if (building.depth == BackToThePast.PARALLAX_2_LAYER) {
					building.x = building.startx + (int) this.world.camera.cameraX / 4;
				}
			}
		}
	}
	
	private void generateBackground(World world) {
		
		// now generate the building entities for our two parallax layers
		generateBuildings(world, worldWidth, worldHeight, BackToThePast.PARALLAX_1_LAYER); // layer 1 is in the back and scrolls slower!
		generateBuildings(world, worldWidth, worldHeight, BackToThePast.PARALLAX_2_LAYER); // layer 2 is in front of layer 1 and scrolls faster!
	}

	private void generateBuildings(World world, float w, float h, int layer) {
		int generatedwidth = (int) w;
		int nrBuildings = buildingImages.size();
		int yoff = 0;
		
		if (layer == BackToThePast.PARALLAX_2_LAYER) {
			generatedwidth = (int) ((float) w * 1.5);
			yoff = 0;
		} else if (layer == BackToThePast.PARALLAX_1_LAYER) {
			generatedwidth = (int) ((float) w * 1.25);
			yoff = - Globals.RND.getRandomInt(10, 60);
		}
		
		int x = -20 + Globals.RND.getRandomInt(-10, 10);
		while (x < generatedwidth) {
//			Log.debug("generateBuildings: w = " + w + ", h = " + h + ", layer = " + layer + ", x = " +x);
			int index = Globals.RND.getRandomInt(0, nrBuildings-1);
			Image img = buildingImages.get(index);
			int y = (int)h - img.getHeight() + Globals.RND.getRandomInt(0, 70);
			y += yoff;
			if (y + img.getHeight() < h)
				y = (int)h - img.getHeight();
			Building building = new Building(x, y, img, layer, false);
			world.add(building);
			x += img.getWidth() + Globals.RND.getRandomInt(-100, 100);
		}
	}
}
