package com.rightanglegames.backtothepast.effects;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.util.Log;

import com.rightanglegames.backtothepast.BackToThePast;

import it.randomtower.engine.ME;
import it.randomtower.engine.entity.Entity;

public abstract class Effect extends Entity {
	
	public static final int UNDEFINED = -1;
	
	private AngelCodeFont font;
	private String[] text;
	
	private Entity toFollow = null;
	private Vector2f offsetOrPosition = null;
	private int duration = -1;
	private int frequency = -1;
	private boolean alphaFade = false;
	private boolean looping = false;
	
	private int lifetime = 0;
	private int frameTicks = 0;
	private int frame = 0;
	private int nrOfFrames = 0;
	private int framesPerRow = 0;
	private int frameInRow = 0;
	private int row = 0;
	private float alphaDecreasePerFrequency = 0;

	
	public Effect(SpriteSheet sheet, int framesOfSheet, Entity toFollow, Vector2f offsetOrPosition, Vector2f speed, int duration,
			int frequency, boolean alphaFade, boolean looping) {
		super(0,0);
		this.sheet = sheet;
		this.framesPerRow = sheet.getHorizontalCount();
		this.nrOfFrames = framesOfSheet;
		this.initialize(toFollow, offsetOrPosition, speed, duration, frequency, alphaFade, looping);
	}

	public Effect(Image image, Entity toFollow, Vector2f offsetOrPosition, Vector2f speed, int duration,
			int frequency, boolean alphaFade, boolean looping) {
		super(0,0);
		this.currentImage = image;
		this.nrOfFrames = 1;
		this.initialize(toFollow, offsetOrPosition, speed, duration, frequency, alphaFade, looping);
	}
	
	public Effect(AngelCodeFont font, String[] text,Entity toFollow, Vector2f offsetOrPosition,
			Vector2f speed, int duration, boolean alphaFade) {
		super(0,0);
		this.font = font;
		this.text = text;
		this.nrOfFrames = 1;
		this.initialize(toFollow, offsetOrPosition, speed, duration, UNDEFINED, alphaFade, false);
	}
	
	private void initialize(Entity toFollow, Vector2f offsetOrPosition, Vector2f speed, int duration, int frequency, boolean alphaFade, boolean looping) {
		this.depth = BackToThePast.EFFECTS_LAYER;
		if (toFollow == null) {
			this.toFollow = null;
			this.x = offsetOrPosition.x;
			this.y = offsetOrPosition.y;
		} else {
			this.toFollow = toFollow;
			this.x = toFollow.x + offsetOrPosition.x;
			this.y = toFollow.y + offsetOrPosition.y;
		}
		this.offsetOrPosition = offsetOrPosition;
		this.speed = speed;
		this.duration = duration;
		this.frequency = frequency;
		this.alphaFade = alphaFade;
		this.looping = looping;

		if (this.frequency != UNDEFINED && this.duration == UNDEFINED) {
			this.duration = this.nrOfFrames * this.frequency;
		} else if (this.duration != UNDEFINED && this.frequency == UNDEFINED) {
			this.frequency = this.duration / this.nrOfFrames;
		}
		if (alphaFade && !looping)
			alphaDecreasePerFrequency = (float) this.frequency / (float) this.duration;
		
	}

	public void update(GameContainer container, int delta) throws SlickException {
		if (ME.useDeltaTiming) {
			lifetime += delta;
			frameTicks += delta;
		} else {
			lifetime++;
			frameTicks++;
		}
		if (lifetime > duration) {
			this.destroy();
			return;
		}
		
		if (toFollow != null) {
			this.x = toFollow.x + offsetOrPosition.x;
			this.y = toFollow.y + offsetOrPosition.y;
		}
		
		if (frameTicks >= frequency) {
			frameTicks -= frequency; 
			frame ++;
			frameInRow++;
			if (frameInRow >= framesPerRow) {
				row++;
				frameInRow = 0;
			}
//			Log.debug("Effect: frame = " + frame + ", frameInRow = " + frameInRow + ", row = " + row);
			if (frame >= nrOfFrames) {
				if (looping) {
					frame = 0;
					frameInRow = 0;
					row = 0;
				} else {
					this.destroy();
					return;
				}
			}
			if (alphaFade)
				this.setAlpha(this.getAlpha() - alphaDecreasePerFrequency);
		}
		if (sheet != null) {
			this.currentImage = this.sheet.getSprite(frameInRow, row);
		}
		super.update(container, delta);
	}

	@Override
	public void render(GameContainer container, Graphics g)	throws SlickException {
		super.render(container, g);
		if (text != null) {
			// let's render the text
			g.setFont(font);
			g.setColor(Color.white);
			g.drawString(text[0], x, y);
		}
	}
	
}
