package com.rightanglegames.backtothepast.enemies;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Vector2f;

import com.rightanglegames.backtothepast.Globals;
import com.rightanglegames.backtothepast.weapons.Fireball;

public class DummyRobot extends Enemy {
	
	public static final String DUMMYROBOT = "dummyrobot";
	
	protected boolean checkGround = true;
	protected Image leftImg, rightImg;
	
	public DummyRobot(float x, float y) {
		super(x, y, 100, 0);
		leftImg = ResourceManager.getImage(DUMMYROBOT);
		rightImg = leftImg.getFlippedCopy(true, false);
		currentImage = leftImg;
		speed = new Vector2f(SPEED, 0);
		width = 40;
		height = 40;
		setHitBox(6, 2, 24, 38);
	}

	public void update(GameContainer container, int delta) throws SlickException {
		if (state == EnemyState.STUNNED) {
			// do nothing. wait until an alarm wakes us up again
		} else if (state == EnemyState.WANDER) {
			super.update(container, delta);
			stayInWorld(true, true);
			if (checkGround)
				checkGround(true, true);
			if (this.canSee(Globals.player, true, false)) {
				state = EnemyState.AGGRESSIVE;
				canShoot = true;
			}
		} else if (state == EnemyState.AGGRESSIVE) {
			super.update(container, delta);
			stayInWorld(true, true);
			if (checkGround)
				checkGround(true, true);
			if (canShoot) {
				canShoot = false;
				shootAt(Globals.player);
				this.setAlarm("shoot", SHOOTTIME, true, true);
			}
			if (!this.canSee(Globals.player, true, false)) {
				state = EnemyState.WANDER;
			}
		}
		if (speed.x > 0)
			this.faceRight = true;
		else
			this.faceRight = false;
		if (this.faceRight)
			currentImage = rightImg;
		else
			currentImage = leftImg;
	}
	
	@Override
	public void collisionResponse(Entity other) {
		if (other.isType(Fireball.FIREBALL)) {
			// let's turn into the fireball direction
			if (other.x < this.x) {
				this.speed.x = -SPEED;
				faceRight = false;
			} else {
				this.speed.x = SPEED;
				faceRight = true;
			}
			// dummy robots are stunned for a second if they are hit by a firewall
			this.state = EnemyState.STUNNED;
			this.setAlarm("stunned", STUNNEDTIME, true, true);
			super.collisionResponse(other);
		} else {
			// try to move in old direction
			if (faceRight && this.speed.x == 0) {
				this.speed.x = -SPEED;
				faceRight = false;
			} else if (!faceRight && this.speed.x == 0){
				this.speed.x = SPEED;
				faceRight = true;
			}
		}
	}
	
	@Override
	public void alarmTriggered(String name) {
		super.alarmTriggered(name);
		if (name.equals("stunned")) {
			// wake up after being hit
			this.state = EnemyState.AGGRESSIVE;
			if (Globals.player.x < this.x)
				this.faceRight = false;
			else
				this.faceRight = true;
		} else if (name.equals("shoot")) {
			canShoot = true;
		}
	}
}
