package com.rightanglegames.backtothepast.enemies;

import it.randomtower.engine.ME;
import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;
import it.randomtower.engine.entity.PhysicsEntity;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Vector2f;

import com.rightanglegames.backtothepast.Acid;
import com.rightanglegames.backtothepast.BackToThePast;
import com.rightanglegames.backtothepast.Bresenham;
import com.rightanglegames.backtothepast.Globals;
import com.rightanglegames.backtothepast.IngameState;
import com.rightanglegames.backtothepast.Level;
import com.rightanglegames.backtothepast.effects.DebrisExplosion;
import com.rightanglegames.backtothepast.effects.Shield;
import com.rightanglegames.backtothepast.effects.Text;
import com.rightanglegames.backtothepast.weapons.Bullet;
import com.rightanglegames.backtothepast.weapons.Bullet.BulletType;
import com.rightanglegames.backtothepast.weapons.Fireball;

public abstract class Enemy extends PhysicsEntity {

	public enum EnemyState { WANDER, STUNNED, DEAD, AGGRESSIVE };
	
	public static final String ENEMY = "enemy";

	protected static float SPEED = 2; 
	protected static int STUNNEDTIME = ME.targetFrameRate / 4;
	protected static int SHOOTTIME = ME.targetFrameRate;
	
	protected boolean faceRight = false;
	protected int health = 100;
	protected int shield = 0;
	protected EnemyState state = EnemyState.WANDER;
	protected Shield shieldEffect = null;
	
	protected Bresenham myEyes;
	protected int sight = 200;
	protected boolean canShoot = true;
	
	protected boolean insultActive = false;
	protected Sound laserSnd;
	
	protected String[] shieldInsults = {
		"Ha! That didn't hurt!",
		"Try again, meat ball!",
		"Now I'll get you!",
		"Is that all?",
		"It's getting hot in here!"
	};
	
	protected String[] hitInsults = {
		"Stop that!",
		"Hey! Stop!",
		"Now you've made me angry!",
		"Ready to rumble!",
		"Who's your daddy?",
		"I'm leaking oil!",
		"Holy bolt! You drive me nuts!",
		"One...two...three!",
		"Help me! He's here!"
	};
	
	
	
	public Enemy(float x, float y, int health, int shield) {
		super(x, y);
		
		this.health = health;
		this.shield = shield;
		this.depth = BackToThePast.ENEMIES_LAYER;
		this.addType(ENEMY);
		this.state = EnemyState.WANDER;
		this.myEyes = new Bresenham();
		this.laserSnd = ResourceManager.getSound("laser");
	}

	public void update(GameContainer container, int delta) throws SlickException {
		super.update(container, delta);
		if (collide(Acid.ACID, x, y) != null) {
			// this robot dies
			this.visible = false;
			this.destroy();
		}
	}
	
	public void stayInWorld(boolean revertHorizontal, boolean revertVertical) {
		if (x < 0) {
			x = 0;
			if (revertHorizontal && speed.x != 0)
				speed.x = -speed.x;
		}
		if (x+width >= world.width) {
			x = world.width - width;
			if (revertHorizontal && speed.x != 0)
				speed.x = -speed.x;
		}
		if (y < 0) {
			y = 0;
			if (revertVertical && speed.y != 0)
				speed.y = -speed.y;
		}
		if (y+height >= world.height) {
			y = world.height - height;
			if (revertVertical && speed.y != 0)
				speed.y = -speed.y;
		}
	}
	
	public void checkGround(boolean revertHorizontal, boolean revertVertical) {
		boolean blocked = ((IngameState) world).getCurrentLevel().blocked(this.x + this.speed.x + ((faceRight)?this.width:0), this.y + this.height + 1);
		if (!blocked) {
			x = previousx;
			y = previousy;
			if (revertHorizontal && speed.x != 0)
				speed.x = -speed.x;
			if (revertVertical && speed.y != 0)
				speed.y = -speed.y;
		}
	}
	
	@Override
	public void collisionResponse(Entity other) {
		if (other.isType(Fireball.FIREBALL)) {
			int damage = ((Fireball) other).getDamage();
			// we take damage
			if (shield > 0) {
				randomShieldInsult();
				if (shieldEffect != null)
					shieldEffect.destroy();
				shieldEffect = new Shield(x,y,this);
				this.world.add(shieldEffect);
				shield -= damage;
				damage = 0;
				if (shield < 0) {
					damage = -shield;
					shield = 0;
				}
			}
			if (damage > 0) {
				health -= damage;
				if (health <= 0) {
					// we are destroyed
					((IngameState)this.world).increaseScore(100);
					this.world.add(new DebrisExplosion(this.x-12, this.y-12,90));
					this.destroy();
					if (shieldEffect != null) {
						shieldEffect.destroy();
						shieldEffect = null;
					}
				}
				randomHitInsult();
			}
		}
	}

	/**
	 * just use tilebased line of sight, using the blocked array of the level
	 * @param target
	 * @param onlyHorizontal - entity and target must be on the same y tile position
	 * @param onlyVertical - entity and target must be on the same x tile position 
	 * @return
	 */
	public boolean canSee(Entity target, boolean onlyHorizontal, boolean onlyVertical) {
		Vector2f myTile = this.getTile(this);
		Vector2f targetTile = this.getTile(target);
		if (onlyHorizontal) {
			if (myTile.y != targetTile.y)
				return false;
		}
		if (onlyVertical) {
			if (myTile.x != targetTile.x)
				return false;
		}
		// is enemy looking in the right direction
		if (this.faceRight && myTile.x > targetTile.x)
			return false;
		if (!this.faceRight && myTile.x < targetTile.x)
			return false;
		// now check with Bresenham if there is nothing solid blocking our view
		Level level = ((IngameState) world).getCurrentLevel();
		int length = myEyes.plot((int)myTile.x, (int)myTile.y, (int)targetTile.x, (int)targetTile.y) * BackToThePast.TILESIZE;
		if (length > sight)
			return false;	// too far away!
		while (myEyes.next()) {
			int linex = myEyes.getX();
			int liney = myEyes.getY();
			if (level.blocked(null, linex, liney))
				return false;
		}
		return true;
	}
	
	protected Vector2f getTile(Entity entity) {
		int tilex, tiley;
		if (entity.centered) {
			tilex = (int) (entity.x / BackToThePast.TILESIZE);
			tiley = (int) (entity.y / BackToThePast.TILESIZE);
		} else {
			tilex = (int) ((entity.x + entity.width/2) / BackToThePast.TILESIZE);
			tiley = (int) ((entity.y + entity.height/2) / BackToThePast.TILESIZE);
		}
		return new Vector2f(tilex, tiley);
	}
	
	public void shootAt(Entity target) {
		int offsetx, bulletspeed;
		if (faceRight) {
			offsetx = 20;
			bulletspeed = 6;
		} else {
			offsetx = -20;
			bulletspeed = -6;
		}
		Bullet bullet = new Bullet(BulletType.YELLOW, x+offsetx, y+10, 60, new Vector2f(bulletspeed, 0), 90);
		this.world.add(bullet);
		laserSnd.play();
	}
	
	
	public void randomShieldInsult() {
		randomInsult(shieldInsults);
	}
	
	public void randomHitInsult() {
		randomInsult(hitInsults);
	}
	
	public void randomInsult(String[] list) {
		if (insultActive)
			return;
		int which = Globals.RND.getRandomInt(0, list.length-1);
		AngelCodeFont font = ResourceManager.getAngelCodeFont("neuropolxfree12");
		String[] text = { list[which] };
		int length = font.getWidth(list[which]);
		int yoff = -(font.getHeight(list[which])+2);
		int xoff = -(length / 2 - this.width / 2);
		this.world.add(new Text(font, text, this, new Vector2f(xoff, yoff), null, BackToThePast.TARGET_FRAME_RATE*3, false));
		this.setAlarm("insult", BackToThePast.TARGET_FRAME_RATE*3, true, true);
		insultActive = true;
	}
	
	@Override
	public void alarmTriggered(String name) {
		if (name.equals("insult"))
			insultActive = false;
	}
}
