package com.rightanglegames.backtothepast.enemies;

import it.randomtower.engine.ME;
import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.util.pathfinding.Mover;
import org.newdawn.slick.util.pathfinding.Path;

import com.rightanglegames.backtothepast.BackToThePast;
import com.rightanglegames.backtothepast.Bresenham;
import com.rightanglegames.backtothepast.Globals;
import com.rightanglegames.backtothepast.IngameState;
import com.rightanglegames.backtothepast.Level;
import com.rightanglegames.backtothepast.Waypoint;
import com.rightanglegames.backtothepast.effects.Shield;
import com.rightanglegames.backtothepast.enemies.Enemy.EnemyState;
import com.rightanglegames.backtothepast.weapons.Bullet;
import com.rightanglegames.backtothepast.weapons.Fireball;
import com.rightanglegames.backtothepast.weapons.Bullet.BulletType;

//TODO HIER WEITER
//wenn aggressiv dann neuen Pfad zum Spieler ermitteln. Wenn aggressiv und spieler nicht zu sehen dann pfad zum
//spieler folgen. sobald sichtbar, anhalten und feuern. wenn spieler nicht mehr sichtbar neuen pfad zu altem waypoint
//ermitteln und weiterfliegen

public class FlyingRobot extends Enemy implements Mover {

	public static final String FLYINGROBOT = "flyingrobot";

	private boolean randomWaypoints = false;
	private Waypoint[] waypoints = null;

	private int indexWaypoints = -1;
	/** the next waypoint this robot is flying to */
	private Waypoint targetWaypoint = null;
	/** the path to the target waypoint */
	private Path targetPath = null;
	/** the index into the path array of tile coordinates */
	private int indexPath = 0;
	/** the tile coordinates of the next path segment */
	private int pathx, pathy;
	/** animation frame, 0 or 1 */
	private int frame = 0;

	private boolean standStill = false;
	
	public FlyingRobot(float x, float y) {
		this(x, y, false);
	}
	
	public FlyingRobot(float x, float y, boolean randomWaypoints) {
		super(x, y, 100, 100);
		sheet = ResourceManager.getSpriteSheet(FLYINGROBOT);
		currentImage = sheet.getSprite(frame, 0);
		width = 40;
		height = 40;
		setHitBox(0, 0, 40, 40);
		this.addType(FLYINGROBOT);
		this.gravity = 0;
		this.friction = new Vector2f(0,0);
		this.stopOnCollision = false;
		this.setAlarm("frame", BackToThePast.TARGET_FRAME_RATE / 5, false, true);
	}
	
	public void update(GameContainer container, int delta) throws SlickException {
		if (waypoints == null)
			prepareWaypoints(randomWaypoints);
		
		if (state == EnemyState.STUNNED) {
			// do nothing. wait until an alarm wakes us up again
		} else if (state == EnemyState.WANDER) {
			super.update(container, delta);
			stayInWorld(true, true);
			followPath();
			if (this.canSee(Globals.player, false, false)) {
				state = EnemyState.AGGRESSIVE;
				canShoot = true;
				targetMage();
			}
		} else if (state == EnemyState.AGGRESSIVE) {
			super.update(container, delta);
			stayInWorld(true, true);
			followMage();
			if (canShoot && this.canSee(Globals.player, false, false)) {
				canShoot = false;
				shootAt(Globals.player);
				this.setAlarm("shoot", SHOOTTIME, true, true);
			}
			if (this.getDistance(Globals.player) > (sight * 1.5f)) {
				// switch back to wander mode
				state = EnemyState.WANDER;
				targetCurrentWaypoint();
			}
		}
		if (speed.x > 0)
			this.faceRight = true;
		else
			this.faceRight = false;
		if (this.faceRight)
			currentImage = sheet.getSprite(frame, 0).getFlippedCopy(true, false);
		else
			currentImage = sheet.getSprite(frame, 0);
	}
	
	@Override
	public void render(GameContainer container, Graphics g)	throws SlickException {
		super.render(container, g);
		// render the path for debugging purposes
		if (ME.debugEnabled) {
			for (int i = indexPath; i < targetPath.getLength(); i++) {
				int x = targetPath.getX(i) * BackToThePast.TILESIZE;
				int y = targetPath.getY(i) * BackToThePast.TILESIZE;
				g.setColor(Color.pink);
				g.drawOval(x, y, BackToThePast.TILESIZE, BackToThePast.TILESIZE);
			}
		}
	}
	
	@Override
	public void collisionResponse(Entity other) {
		if (other.isType(Fireball.FIREBALL)) {
//			// let's turn into the fireball direction
//			if (other.x < this.x) {
//				this.speed.x = -SPEED;
//				faceRight = false;
//			} else {
//				this.speed.x = SPEED;
//				faceRight = true;
//			}
			// even flying robots are stunned for a second if they are hit by a firewall
			this.state = EnemyState.STUNNED;
			this.setAlarm("stunned", STUNNEDTIME, true, true);
			super.collisionResponse(other);
		}
	}
	
	@Override
	public void alarmTriggered(String name) {
		super.alarmTriggered(name);
		if (name.equals("stunned")) {
			// wake up after being hit
			this.state = EnemyState.AGGRESSIVE;
//			if (Globals.player.x < this.x)
//				this.faceRight = false;
//			else
//				this.faceRight = true;
			canShoot = true;
			targetMage();
		} else if (name.equals("shoot")) {
			canShoot = true;
		} else if (name.equals("frame")) {
			frame++;
			if (frame > 1)
				frame = 0;
		}
	}

	@Override
	public void shootAt(Entity target) {
		int offsetx, bulletspeed = 6;
		if (faceRight) {
			offsetx = 20;
		} else {
			offsetx = -20;
		}
		Vector2f aimAndSpeed = new Vector2f(target.x - x, target.y - y);
		aimAndSpeed.normalise().scale(bulletspeed);
		Bullet bullet = new Bullet(BulletType.YELLOW, x+offsetx, y+10, 60, aimAndSpeed, 90);
		this.world.add(bullet);
	}


	private void prepareWaypoints(boolean randomWaypoints) {
		int waypointsInLevel = ((IngameState) world).getCurrentLevel().getNrOfWaypoints();
		if (!randomWaypoints) {
			waypoints = new Waypoint[waypointsInLevel];
			// work through them in the given order but start with a random one
			for (int i = 0; i < waypointsInLevel; i++)
				waypoints[i] = ((IngameState) world).getCurrentLevel().getWaypoints()[i].copy();
		} else {
			//TODO implement
		}
		startWithNearestWaypoint();
	}
	
	private void startWithNearestWaypoint() {
		indexWaypoints = 0;
		float distance = Float.MAX_VALUE;
		
		Vector2f myPos = new Vector2f(x,y);
		for (int i = 0; i < waypoints.length; i++) {
			Vector2f waypointPos = waypoints[i].getTilepos();
			if (waypointPos.distance(myPos) < distance)
				indexWaypoints = i;
		}
		targetWaypoint = waypoints[indexWaypoints];
		targetCurrentWaypoint();
	}
	
	private void targetNextWaypoint() {
		indexWaypoints ++;
		if (indexWaypoints >= waypoints.length)
			indexWaypoints = 0;
		targetWaypoint = waypoints[indexWaypoints];
		targetCurrentWaypoint();
	}
	
	private void targetCurrentWaypoint() {
		targetPath = ((IngameState) world).getPathfinder().findPath(this, (int) this.getTile(this).x,
				(int) this.getTile(this).y, (int) targetWaypoint.getTilepos().x, (int) targetWaypoint.getTilepos().y);
		indexPath = -1;
		targetNextPathSegmentCoordinates();
	}
	
	private void targetMage() {
		// find a new path to the player and follow that one
		targetPath = ((IngameState) world).getPathfinder().findPath(this, (int) this.getTile(this).x,
				(int) this.getTile(this).y, (int) this.getTile(Globals.player).x, (int) this.getTile(Globals.player).y);
		if (targetPath == null) {
			state = EnemyState.WANDER;
			targetNextWaypoint();
			return;
		}
		indexPath = -1;
		targetNextPathSegmentCoordinates();
	}
	
	private void targetNextPathSegmentCoordinates() {
		indexPath++;
		if (indexPath >= targetPath.getLength() && state == EnemyState.AGGRESSIVE) {
			// mage has moved! retarget!
			targetMage();
			return;
		}
		pathx = targetPath.getX(indexPath);
		pathy = targetPath.getY(indexPath);
		int deltax = pathx - (int)this.getTile(this).x;
		int deltay = pathy - (int)this.getTile(this).y;
		while (deltax == 0 && deltay == 0) {
			indexPath++;
			pathx = targetPath.getX(indexPath);
			pathy = targetPath.getY(indexPath);
			deltax = pathx - (int)this.getTile(this).x;
			deltay = pathy - (int)this.getTile(this).y;
		}
		speed = new Vector2f(deltax, deltay).normalise().scale(SPEED);
	}
	
	private void followPath() {
		// did we reach the waypoint
		float pointx = targetWaypoint.getTilepos().x * BackToThePast.TILESIZE;
		float pointy = targetWaypoint.getTilepos().y * BackToThePast.TILESIZE;
		if (Math.abs(pointx - this.x) <= SPEED && Math.abs(pointy - this.y) <= SPEED) {
			this.x = pointx;
			this.y = pointy;
			// yes, target next waypoint
			targetNextWaypoint();
		} else {
			pointx = pathx * BackToThePast.TILESIZE;
			pointy = pathy * BackToThePast.TILESIZE;
			if (Math.abs(pointx - this.x) <= SPEED && Math.abs(pointy - this.y) <= SPEED) {
				// yes
				this.x = pointx;
				this.y = pointy;
				targetNextPathSegmentCoordinates();
			}
		}
	}
	
	private void followMage() {
		float pointx = pathx * BackToThePast.TILESIZE;
		float pointy = pathy * BackToThePast.TILESIZE;
		if (Math.abs(pointx - this.x) <= SPEED && Math.abs(pointy - this.y) <= SPEED) {
			// yes
			this.x = pointx;
			this.y = pointy;
			targetNextPathSegmentCoordinates();
		}
	}
	
	
}
