package com.rightanglegames.backtothepast.weapons;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;
import org.newdawn.slick.geom.Vector2f;

import com.rightanglegames.backtothepast.Crate;
import com.rightanglegames.backtothepast.effects.DebrisExplosion;
import com.rightanglegames.backtothepast.effects.Explosion;
import com.rightanglegames.backtothepast.enemies.Enemy;

import it.randomtower.engine.ResourceManager;
import it.randomtower.engine.entity.Entity;

public class Fireball extends Weapon {

	public static final String FIREBALL = "fireball";
	private int frameCount = 0;
	private int nextImageFrame = 20;
	private int frame = 0;
	private boolean faceRight;
	private Sound explosionSnd, wallExplosionSnd;
	
	public Fireball(float x, float y, int damage, Vector2f speed, float life) {
		super(x, y, damage, speed, life);
		sheet = ResourceManager.getSpriteSheet(FIREBALL);
		explosionSnd = ResourceManager.getSound("explosion");
		wallExplosionSnd = ResourceManager.getSound("wallexplosion");
		this.addType(FIREBALL);
		speed.y = 0;
		if (speed.x < 0)
			faceRight = false;
		else
			faceRight = true;
		if (faceRight) {
			currentImage = sheet.getSprite(frame, 0);
		} else {
			currentImage = sheet.getSprite(frame, 0).getFlippedCopy(true, false);
		}
		this.setHitBox(0, 4, 64, 16);
	}
	
	public void update(GameContainer container, int delta) throws SlickException {
		float explx;
		
		super.update(container, delta);
		frameCount++;
		if (frameCount > nextImageFrame) {
			frame++;
			if (frame > 31)
				frame = 0;
			if (faceRight)
				currentImage = sheet.getSprite(frame, 0);
			else
				currentImage = sheet.getSprite(frame, 0).getFlippedCopy(true, false);
		}
		// handle speed, only horizontal movement
//		this.x += speed;
		// collision with robot/enemy - deal damage
		Entity enemy = collide(Enemy.ENEMY, x, y);
		if (enemy != null) {
			explosionSnd.play();
			// enemy takes damage and we die in an explosion
			if (faceRight)
				explx = this.x + this.width + 24;
			else
				explx = this.x -24;
			this.world.add(new DebrisExplosion(explx, this.y-12,60));
			this.destroy();
			return;
		}
		// collision with crates - destroy this and crate in an explosion
		Entity crate = collide(Crate.CRATE, x, y);
		if (crate != null) {
			explosionSnd.play();
			if (faceRight)
				explx = this.x + this.width + 24;
			else
				explx = this.x -24;
			this.world.add(new DebrisExplosion(explx, this.y-12,60));
			this.destroy();
			return;
		}
		// collision with solid - destroy and create explosion
		Entity solid = collide(Entity.SOLID, x, y);
		if (solid != null && !solid.isType(PLAYER)) {
			wallExplosionSnd.play();
			if (faceRight)
				explx = this.x + this.width + 24;
			else
				explx = this.x - 24;
			this.world.add(new Explosion(explx, this.y-12, 60));
			this.destroy();
		}
	}

}
